#include "Arduino.h"
#include "Encoder.h" 
#include "PID_v1.h"
#include "DFRobotHCRProtocol.h"
#include "Wire.h"
#include <ArduinoJson.h>
int box_z = -1;
//int box_x = -1;
int letrun = -1;
String json = "";

//------------------------------| define |----------------------------------------------------------------------------------------------------------------------------

#define CRITICAL_SECTION_START  unsigned char _sreg = SREG; cli();
#define CRITICAL_SECTION_END    SREG = _sreg;
#define SAMPLE_TIME_MS 5
#define WHEEL_MAX_SPEED 50 //50cm/s

#define LEFT_SPEED_PIN 9 // Prototypes
#define LEFT_DIRECTION_PIN 8 // Prototypes
#define RIGHT_SPEED_PIN 10
#define RIGHT_DIRECTION_PIN 11

#define ENCODER_LEFT_A 2
#define ENCODE#define ENCODER_RIGHT_B 5

#define KP 2.0
#define KI 1.0
#define KD 0.1

#define WHEEL_PERIMETER_CM (13.5*PI)  //42.41025
#define ENCODER_PPR (663*4)  //2652
#define WHEEL_CM_PER_ENCODER_PPR (WHEEL_PERIMETER_CM/ENCODER_PPR)  // 0.015991799

#define DIRECTION_STOP 0
#define DIRECTION_FORWARD 1
#define DIRECTION_BACKWARD 2

//------------------------| 设置变量 |----------------------------------------------------------------------------------------------------------------------------------

DFRobotHCRProtocol mDFRobotHCRProtocol; // Define variables and constants
Encoder encoderLeft(ENCODER_LEFT_A, ENCODER_LEFT_B);
Encoder encoderRight(ENCODER_RIGHT_A, ENCODER_RIGHT_B);

static const uint16_t PWM_FREQUENCY = 20000; // The motor driver can handle a PWM frequency up to 20kHz
static const int PWMVALUE = F_CPU / PWM_FREQUENCY / 2; // The frequency is given by F_CPU/(2*N*ICR) - where N is the prescaler, prescaling is used so the frequency is given by F_CPU/(2*ICR) - ICR = F_CPU/PWM_FREQUENCY/2

double setPointLeft=0.0, inputLeft=0.0, outputLeft=0.0;
double setPointRight=0.0, inputRight=0.0, outputRight=0.0;//Define Variables we'll be connecting to

unsigned long speedTimer = millis();

PID pidLeft(&inputLeft, &outputLeft, &setPointLeft,KP,KI,KD, DIRECT);
PID pidRight(&inputRight, &outputRight, &setPointRight,KP,KI,KD, DIRECT); //Specify the links and initial tuning parameters

uint8_t lastDirectionLeft=DIRECTION_STOP;
uint8_t lastDirectionRight=DIRECTION_STOP;

float speedLeft = 0.0;
float speedRight = 0.0;

float targetSpeedLeft = 0.0;
float targetSpeedRight = 0.0;

float acceleration= 0.005;
float deacceleration = 0.005;

//---| 自定义函数 |-------------------------------------------------------------------------------------------------------------------------------------------------------

//-------------------setSpeed------------------------------------------------------------------------------------------------

void setSpeed(float theLeft, float theRight){ // Encoder Signal Number / second
  theLeft /= WHEEL_CM_PER_ENCODER_PPR*1000;
  theRight /= WHEEL_CM_PER_ENCODER_PPR*1000;
  
  //
  theLeft = constrain(theLeft, -2.0, 2.0);
  theRight = constrain(theRight, -3.0, 3.0);
  
  targetSpeedLeft = theLeft;
  targetSpeedRight = theRight;
}  //-150 ~ +150

//-------------------Motor_setup----------------------------------------------------------------------------------------------

void Motor_setup()
{
  pinMode(LEFT_DIRECTION_PIN, OUTPUT);
  pinMode(RIGHT_DIRECTION_PIN, OUTPUT);
  
  pinMode(LEFT_SPEED_PIN, OUTPUT);
  pinMode(RIGHT_SPEED_PIN, OUTPUT);
  
  TCCR1A=_BV(COM1A1) | _BV(COM1B1);
  TCCR1B=_BV(WGM13) | _BV(CS10);
  
  
  OCR1A = 0;
  OCR1B = 0;
  
  //16000000/2/1/20000
  CRITICAL_SECTION_START;
  ICR1=PWMVALUE;
  CRITICAL_SECTION_END;
  
  //  Serial.println(PWMVALUE);
}

//------------------setMotorSpeed----------------------------------------------------------------------------------------------

void setMotorSpeed(int left, int right)
{
  left = constrain(left, -PWMVALUE, PWMVALUE);
  right = constrain(right, -PWMVALUE, PWMVALUE);
  
  if (left>=0) {
    digitalWrite(LEFT_DIRECTION_PIN, HIGH);
    OCR1A = left;
  }
  else{
    digitalWrite(LEFT_DIRECTION_PIN, LOW);
    OCR1A = -left;
  }
  
  if (right>=0) {
    digitalWrite(RIGHT_DIRECTION_PIN, HIGH);
    OCR1B = right;
  }
  else{
    digitalWrite(RIGHT_DIRECTION_PIN, LOW);
    OCR1B = -right;
  }
  
}

//------------------updateSpeed------------------------------------------------------------------------------------------------

void updateSpeed(){
  
  unsigned long duration = millis() - speedTimer;
 
  if (duration >= SAMPLE_TIME_MS) {
    if (targetSpeedLeft == speedLeft) {
      setPointLeft += speedLeft * duration;
    }
    else if (targetSpeedLeft > speedLeft) {
      setPointLeft += speedLeft * duration + acceleration * duration * duration /2;
      speedLeft += acceleration * duration;
      if (targetSpeedLeft < speedLeft) {
        speedLeft = targetSpeedLeft;
      }
    }
    else{
      setPointLeft += speedLeft * duration - deacceleration * duration * duration /2;
      speedLeft -= deacceleration * duration;
      if (targetSpeedLeft > speedLeft) {
        speedLeft = targetSpeedLeft;
      }
    }
    
    
    if (targetSpeedRight == speedRight) {
      setPointRight += speedRight * duration;
    }
    else if (targetSpeedRight > speedRight) {
      setPointRight += speedRight * duration + acceleration * duration * duration /2;
      speedRight += acceleration * duration;
      if (targetSpeedRight < speedRight) {
        speedRight = targetSpeedRight;
      }
    }
    else{
      setPointRight += speedRight * duration - deacceleration * duration * duration /2;
      speedRight -= deacceleration * duration;
      if (targetSpeedRight > speedRight) {
        speedRight = targetSpeedRight;
      }
    }
        
    //    setPointRight += duration * speedRight;
    
    //  Serial.print("setPoint: ");
    //  Serial.print(setPointLeft);
    //  Serial.print(", ");
    //  Serial.println(setPointRight);
    //
    //  Serial.print("setPointRight: ");
    //  Serial.println(setPointRight);
    
    inputLeft = -encoderLeft.read();
    
    //  Serial.print("inputLeft: ");
    //  Serial.println(inputLeft);
    
    inputRight = -encoderRight.read();
    
    pidLeft.Compute();
    pidRight.Compute();
    
    
    //    if (outputLeft == 0) {
    //      lastDirectionLeft = DIRECTION_STOP;
    //    }
    //    else if (outputLeft > 0) {
    //      if (lastDirectionLeft != DIRECTION_FORWARD) {
    //        outputLeft = PWMVALUE/1.5;
    //      }
    //      lastDirectionLeft = DIRECTION_FORWARD;
    //    }
    //    else{
    //      if (lastDirectionLeft != DIRECTION_BACKWARD) {
    //        outputLeft = -PWMVALUE/1.5;
    //      }
    //      lastDirectionLeft = DIRECTION_BACKWARD;
    //    }
    //
    //
    //
    //    if (outputRight == 0) {
    //      lastDirectionRight = DIRECTION_STOP;
    //    }
    //    else if (outputRight > 0) {
    //      if (lastDirectionRight != DIRECTION_FORWARD) {
    //        outputRight = PWMVALUE/1.5;
    //      }
    //      lastDirectionRight = DIRECTION_FORWARD;
    //    }
    //    else{
    //      if (lastDirectionRight != DIRECTION_BACKWARD) {
    //        outputRight = -PWMVALUE/1.5;
    //      }
    //      lastDirectionRight = DIRECTION_BACKWARD;
    //    }
    

    setMotorSpeed(outputLeft, outputRight);
    
    speedTimer = millis();
  }
}

//------------------requestEvent---------------------------------------------------------------------------------------

void requestEvent()
{
  while (mDFRobotHCRProtocol.available()) {
    if (mDFRobotHCRProtocol.readCommand() == COMMAND_WHEEL_SPEED) {
      //      int left = mDFRobotHCRProtocol.read();
      //      int right = mDFRobotHCRProtocol.read();
      //      setSpeed(left, right);
    }
  }
  
  //  byte RequestString[Pbn] = {
  //    'F','E',0,0,0,0,0,10  };
  //
  //  RequestString[2] = 0x00;
  //
  //  if(pastCoder[LF] < 0)  bitWrite(RequestString[2],4,1);
  //  if(pastCoder[RT] < 0)  bitWrite(RequestString[2],0,1);
  //  //  Serial.println(RequestString[2],HEX);
  //
  //  RequestString[3] = abs(pastCoder[LF]) / 256;
  //  RequestString[4] = abs(pastCoder[LF]) % 256;
  //  RequestString[5] = abs(pastCoder[RT]) / 256;
  //  RequestString[6] = abs(pastCoder[RT]) % 256;
  //
  //  totalCoder[LF] += pastCoder[LF];
  //  totalCoder[RT] += pastCoder[RT];
  //
  //  pastCoder[LF] = 0;
  //  pastCoder[RT] = 0;
  //
  //  Wire.send(RequestString,Pbn);
  //
  /*
   for( int j = 0;j < Pbn;j++)
   {
   Serial.print(j);
   Serial.print(":");
   Serial.println(RequestString[j],HEX);
   }
   Serial.println();
   */
}

//------------------requestEvent(int HowMany)-------------------------------------------------------------------------

void receiveEvent(int HowMany)
{
  while (mDFRobotHCRProtocol.available()) {
    if (mDFRobotHCRProtocol.readCommand() == COMMAND_MOTOR_SPEED) {
      int left = mDFRobotHCRProtocol.read();
      int right = mDFRobotHCRProtocol.read();
      setSpeed(left, right);
    }
  }
}

//------------------| SETUP |-------------------------------------------------------------------------------------------

void setup()
{
  
  pinMode(LED_BUILTIN, OUTPUT);
  Serial.begin(115200);
  // RealsenseLight
  
  Wire.begin(10);
  //  Wire.onRequest(requestEvent); // register event
  Wire.onReceive(receiveEvent); // register event
  mDFRobotHCRProtocol.begin(Wire);
  
  //  servo1.attach(SERVO_HIGH_PIN);
  //  servo1.write(150);  //30~270 150
  Motor_setup();
 
  //turn the PID on
  pidLeft.SetMode(AUTOMATIC);
  pidRight.SetMode(AUTOMATIC);
  pidLeft.SetOutputLimits(-PWMVALUE, PWMVALUE);
  pidRight.SetOutputLimits(-PWMVALUE, PWMVALUE);
  pidLeft.SetSampleTime(SAMPLE_TIME_MS);
  pidRight.SetSampleTime(SAMPLE_TIME_MS);
  updateSpeed();
//  setMotorSpeed(400,400);
//    delay(3000);
//    setSpeed(10,10);
//    analogWrite(10, 200);
}

int mySpeed = 20;

int state = 1;

//------------------| LOOP |-----------------------------------------------------------------------------------------

void loop()
{
  updateSpeed();

  static unsigned long timer = millis();

  if(millis() - timer >= 0 && millis() - timer <= 200){
    //timer = millis();
      if(state == 1){
        setSpeed(20, 5);
      } 
     
  }

  if(millis() - timer >= 3000 && millis() - timer <= 3200){
    //timer = millis();
      if(state == 1){
        setSpeed(30, 30);
      } 
  }
//
//  if(millis() - timer >= 10000 && millis() - timer <= 10200){
//    //timer = millis();
//      if(state == 1){
//        setSpeed(0, 0);
//      } 
//  }

//  if(millis() - timer >= 5000 && millis() - timer <= 5200){
//    timer = millis();
//      if(state == -1){
//        setSpeed(0, 0);
//        //state = -1;
//      } 
//  }



//        setSpeed(5, 5);
//        delay(3000);
////
//        updateSpeed();
//        setSpeed(0,0);
//        delay(3000);

  
//  static unsigned long timer = millis();
//  if (timer - millis() >1000){
//  }
//------------------ Realsense --------------------------------------------------------------------------------------

//  if(Serial.available() > 0 & Serial) {
//    char t = Serial.read();
//    if(t == '\n')
//    {
//      //json += t;
//      //Serial.println(json);
//      String temp = json;
//      //Serial.println(temp);
//      StaticJsonBuffer<500> jsonBuffer;
//      JsonObject& root = jsonBuffer.parseObject(temp);
//      //delay(1);
//      
//      if(!root.success())
//      {
//        //Serial.println("parse failed");
//        temp = "";
//        json = "";
//        //return;
//      }
//      
//      //box_x = root["tongue"];
//      box_z = root["_index"];
//      //Serial.println(box_x);
//      Serial.println(box_z);
//      
//      //if(box_x > 50)
//      if(box_z == 1)
//      {
//         digitalWrite(LED_BUILTIN, HIGH); 
//         letrun = 1;
//         //setSpeed(20,-20);
//         //delay(500);
//         //box_x = -1;
//         box_z = -1;
//
//         //delay(1000);   
//      }
//      else
//      {
//        digitalWrite(LED_BUILTIN, LOW);
//        letrun = 0;  
//      }
//     
//      temp = "";
//      json = "";
//    }
//    else
//    {
//      json += char(t);
//      //digitalWrite(13, LOW);
//      //delay(2);
//    }
//    
//  }  
  
//------------------ How to Run -------------------------------------------------------------------------------------

//#define LEFT_SPEED_PIN 9 // Prototypes
//#define LEFT_DIRECTION_PIN 8 // Prototypes
//#define RIGHT_SPEED_PIN 10
//#define RIGHT_DIRECTION_PIN 11
//
//digitalWrite(LEFT_DIRECTION_PIN,HIGH);
//analogWrite(LEFT_SPEED_PIN,2);
////delay(2000);
////
//
//
//digitalWrite(RIGHT_DIRECTION_PIN,LOW);
//analogWrite(RIGHT_SPEED_PIN,2);
//
//delay(2000);
//
//analogWrite(LEFT_SPEED_PIN,0);
//
//analogWrite(RIGHT_SPEED_PIN,0);
//delay(2000);


//
//
//
//
//
//
//  if(letrun == 1){
//    setSpeed(20,-10);
// } 
//  
//  else if(letrun == 0){
//    //digitalWrite(LEFT_DIRECTION_PIN, LOW);
//    digitalWrite(LEFT_SPEED_PIN, LOW);
//    //delay(100);
//    //digitalWrite(RIGHT_DIRECTION_PIN, LOW);
//    digitalWrite(RIGHT_SPEED_PIN, LOW);
//
//  }
//
//  else if(letrun == -1){
//    
//     setSpeed(10,10);
//     delay(4867);
//     digitalWrite(LEFT_SPEED_PIN, LOW);
//     digitalWrite(RIGHT_SPEED_PIN, LOW);
//
//
//  }
//
//delay(1000);
//letrun--;
//



//  if(millis())

// setSpeed(2,-1);
//
//delay(2000);
//
//setSpeed(0,0);
//
//delay(2000);


 
 /* 
  digitalWrite(LEFT_DIRECTION_PIN, HIGH);
    digitalWrite(LEFT_SPEED_PIN, HIGH);
    delay(1000);
    digitalWrite(RIGHT_DIRECTION_PIN, HIGH);
    digitalWrite(RIGHT_SPEED_PIN, HIGH);


  while (Serial.available() > 0) {
    
    // look for the next valid integer in the incoming serial stream:
    float p = Serial.parseFloat();
    // do it again:
    float i = Serial.parseFloat();
    // do it again:
    float d = Serial.parseFloat();
    
    // look for the newline. That's the end of your
    // sentence:
    if (Serial.read() == '\n') {
      // constrain the values to 0 - 255 and invert
      // if you're using a common-cathode LED, just use "constrain(color, 0, 255);"
      // print the three numbers in one string as hexadecimal:
      pidLeft.SetTunings(p, i, d);
      pidRight.SetTunings(p, i, d);
    }
  }
  */
  
  
  //#define TIMER timer
  //  static unsigned long timer = millis();
  //  if (millis() - timer >= 3000) {
  //    TIMER = millis();
  //    setSpeed(mySpeed , mySpeed);
  //    mySpeed=-mySpeed;
  //
  //  }
  //#undef TIMER
  
  //#define TIMER updateTimer
  //  static unsigned long TIMER = millis();
  //  if (millis() - TIMER >= 100) {
  //    TIMER = millis();
  //
  //
  //    Serial.print(setPointLeft);
  //    Serial.print(',');
  //    Serial.print(inputLeft);
  //
  ////    Serial.print(inputLeft - setPointLeft);
  //    Serial.print(',');
  //    //  Serial.print(outputLeft);
  //    //  Serial.print(',');
  //    Serial.println(outputLeft);
  //
  //
  //  }
  //#undef TIMER
  
  //  Motor_speed(300, 300);
  //
  //
  //  lastSpeed = leftEncoder.read();
  //  delay(100);
  //
  //  Serial.println(leftEncoder.read() - lastSpeed);
  
  //  for (int i=0; i<PWMVALUE; i++) {
  //    Motor_speed(i, i);
  //    delay(200);
  //
  //
  //  }
  
  //  digitalWrite(A0, HIGH);
  //  digitalWrite(A1, LOW);
  //  delay(3000);
  //  digitalWrite(A0, LOW);
  //  digitalWrite(A1, HIGH);
  //  delay(3000);
  //  


//------------------| YAO YAO |-----------------------------------------------------------------------------------------


//#define LEFT_SPEED_PIN 9 // Prototypes
//#define LEFT_DIRECTION_PIN 8 // Prototypes
//#define RIGHT_SPEED_PIN 10
//#define RIGHT_DIRECTION_PIN 11
//
//digitalWrite(LEFT_DIRECTION_PIN,HIGH);
//digitalWrite(LEFT_SPEED_PIN,HIGH);
////delay(2000);
////
//
//
//digitalWrite(RIGHT_DIRECTION_PIN,HIGH);
//digitalWrite(RIGHT_SPEED_PIN,HIGH);
//
//delay(1000);
//
//digitalWrite(LEFT_SPEED_PIN,LOW);
//
//digitalWrite(RIGHT_SPEED_PIN,LOW);
//delay(1000);
//
//digitalWrite(LEFT_DIRECTION_PIN,LOW);
//digitalWrite(LEFT_SPEED_PIN,HIGH);
////delay(1000);
////
//
//
//digitalWrite(RIGHT_DIRECTION_PIN,LOW);
//digitalWrite(RIGHT_SPEED_PIN,HIGH);
//
//delay(1000);

}
R_LEFT_B 4
#define ENCODER_RIGHT_A 3



